@@ -37,9 +37,12 @@ from docopt import docopt
 from slugify import slugify
 from jinja2 import Environment
 from jinja2 import FileSystemLoader
+from pygments import highlight
+from pygments.lexers import get_lexer_by_name
+from pygments.formatters import get_formatter_by_name
 
 
-VERSION = '14.09.06-dev'
+VERSION = '14.09.07-dev'
 
 
 class Parser:
@@ -279,7 +282,10 @@ class HTMLNoteBuilder:
                             line = line.replace('---', '—')
                             line = line.replace('--', '–')
                             if self.last_is_inline:
-                                output += ' %s' % line
+                                if empty_line_sequence_size:
+                                    output += ' %s' % line
+                                else:
+                                    output += '%s' % line
                             else:
                                 output += '<p>%s' % line
                                 self.last_is_inline = True
@@ -359,13 +365,30 @@ class HTMLNoteBuilder:
         self.last_is_inline = True
         return '<code>%s</code>' % text
 
-    def include(self, filepath):
+    def include(self, filepath, lang=None):
         self.last_is_inline = False
         file = filepath
         with open(file) as f:
             output = f.read()
-        output = escape(output)
-        return '<div class="hll"><pre>%s</pre></div>' % output
+        if lang:
+            html = get_formatter_by_name('html')
+            lexer = get_lexer_by_name(lang)
+            output = highlight(output, lexer, html)
+            # remove wrapping div to insert legend
+            start = len('<div class="highlight">')
+            end = -len('</div>\n')
+            output = output[start:end]
+        else:
+            output = escape(output)
+            output = '<pre>%s</pre>' % output
+        icon = '<span class="glyphicon glyphicon-cloud-download"></span>'
+        legend = '<a class="legend" href="%s">%s <code>%s</code></a>' % (
+            filepath,
+            icon,
+            filepath,
+        )
+        output = '<div class="highlight">%s%s</div>' % (output, legend)
+        return output
 
     def blockdiag(self, name, command):
         self.last_is_inline = False
@@ -388,8 +411,8 @@ class HTMLNoteBuilder:
         filepath = os.path.abspath(filepath)
         command = 'pygmentize -f html %s' % filepath
         output = check_output(command.split())
-        output = output.decode('utf-8')
-        return '<div class="hll">%s</div>' % output
+        output = output.decode('utf')
+        return output
 
 
 def new_draft(title):
@@ -409,21 +432,20 @@ def new_draft(title):
 
 
 def render_html(path):
-    filepath = os.path.join(path, 'index.azf')
-    json = dung(filepath)
-    # * Render body and retrieve a few variables
     # move curdir to the note directory
     # because the note file is written in the
     # note file perspective
     origin_directory = os.curdir
     origin_directory = os.path.abspath(origin_directory)
-    directory = os.path.dirname(path)
+    directory = os.path.abspath(path)
     os.chdir(directory)
+    index = os.path.abspath('index.azf')
+    json = dung(index)
+    # * Render body and retrieve a few variables
     # translate
     render = HTMLNoteBuilder().render
     context = render(json)
     os.chdir(origin_directory)
-
     # render note with template
     now = datetime.now()
     created_at = now.year, now.month, now.day
@@ -450,26 +472,89 @@ def render_html(path):
     return context
 
 
+def render_jinja2(template, context, output):
+    templates = os.path.abspath(os.curdir)
+    templates = os.path.join(templates, '_templates')
+    environment = Environment(
+        loader=FileSystemLoader(templates),
+    )
+    environment.filters['slugify'] = slugify
+    template = environment.get_template(template)
+    out = template.render(**context)
+    with open(output, 'w') as f:
+        f.write(out)
+
+
+def new_book(title):
+    text = """ⵣtitle{%s}
+
+There is the new book!
+
+ⵣbegin_toc
+* path/to/part-one
+* path/to/part-two
+ⵣend_toc
+"""
+    text = text % title
+    with open('index.azf', 'w') as f:
+        f.write(text)
+
+def new_book_part(title):
+    text = """ⵣtitle{%s}
+
+There is the new part of the book!
+
+ⵣbegin_toc
+* path/to/chapter-one
+* path/to/chapter-two
+ⵣend_toc
+"""
+    text = text % title
+    slug = slugify(title)
+    os.makedirs(slug)
+    index = os.path.join(slug, 'index.azf')
+    with open(index, 'w') as f:
+        f.write(text)
+
+
+def new_book_chapter(part, title):
+    text = """ⵣtitle{%s}
+
+A full chapter!
+"""
+    text = text % title
+    slug = slugify(title)
+    path = os.path.join(part, slug)
+    os.makedirs(path)
+    index = os.path.join(path, 'index.azf')
+    with open(index, 'w') as f:
+        f.write(text)
+
+
 def main():
     doc = """main.py.
 
 Usage:
+  main.py new book <title>
+  main.py new book part <title>
+  main.py new book chapter <part> <title>
   main.py new draft <title>
   main.py publish note <path>
   main.py publish page <path>
   main.py dung <path> [<output>] [--pretty-print=<indent>] [--ugly]
   main.py render html <path>
+  main.py render jinja2 <template> <context> <output>
   main.py build
   main.py -h | --help
   main.py --version
 
 Options:
   --pretty-print=<indent> Pretty print indentation [default: 4].
-  --ugly       Ugly JSON.
-  -h --help         Show this screen.
-  --version         Show version.
+  --ugly                  Ugly JSON.
+  -h --help               Show this screen.
+  --version               Show version.
 """
-    arguments = docopt(doc, version='14.09-dev Pyramid of Cheops')
+    arguments = docopt(doc, version='14.09.07-dev Pyramid of Cheops')
     if os.environ.get('DEBUG', False):
         print(arguments)
     # dispatch
@@ -489,10 +574,28 @@ Options:
         path = arguments['<path>']
         render_html(path)
         print('Done!')
+    elif arguments['render'] and arguments['jinja2']:
+        template = arguments['<template>']
+        context = arguments['<context>']
+        output = arguments['<output>']
+        with open(context) as f:
+            context = loads(f.read())
+        render_jinja2(template, context, output)
+        print('Done!')
     elif arguments['new'] and arguments['draft']:
         title = arguments['<title>']
         new_draft(title)
         print('Done!')
+    elif arguments['new'] and arguments['book']:
+        title = arguments['<title>']
+        if arguments['part']:
+            new_book_part(title)
+        elif arguments['chapter']:
+            part = arguments['<part>']
+            new_book_chapter(part, title)
+        else:
+            new_book(title)
+        print('Done!')
     elif arguments['publish'] and arguments['note']:
         path = arguments['<path>']
         context = render_html(path)
@@ -575,6 +678,7 @@ Options:
             notes=notes,
             title='Wellcome',
             bodyclass='index',
+            keywords=keywords.keys(),
         )
         # save
         with open('index.html', 'w') as f:
